; 
; File Name		:'ISR.asm"
; Title			:8515 Interrupt Handlers
; Date			:
; Version		:
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Target MCU		:AT90S8515
;
; DESCRIPTION
; All interrupt handlers reside here
;
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.07.29  dvh	Creation
;	0.02	98.08.20  dvh	Added Timer 1 int for servo control, a little cleanup
;				and better documentation elsewhere
;	0.03	98.08.21  dvh	Added support in TIM0_OVF for a FRAME_DELAY
;				variable used in SERVO.ASM. 
;				Converted Frame_Delay to SRAM from register
;	0.04	98.08.23  dvh	Regularized to XL,XH notation instead of R26.R27 etc
;	0.05	99.09.09  dvh	Fixed morse handler to work properly.
;	0.06	99.09.13  dvh	Modified to work with new circular buffer
;	0.07	99.09.17  dvh	Many updates to UDRIE, RXC and T0 to accomodate
;				full duplex serial comms, with hardware and 
;				xon/xoff handshaking. Hardware working and tested
;				xon not tested yet.
;	0.08	99.10.20  dvh	Added EEPROM_Timer in T0 ISR
;***************************************************************************
;Fixed jump vectors
;
;$000
	rjmp	INIT_Machine	;Restart vector, points to the beginning of the code. 
;$001
	rjmp	EXT_INT0	;IRQ 0
;$002
	rjmp	EXT_INT1	;IRQ 1 
;$003	
	rjmp	TIM1_CAPT	;Timer 1 capture 
;$004
	rjmp	TIM1_COMPA	;Timer 1 compare A
;$005
	rjmp	TIM1_COMPB	;Timer 1 compare B
;$006
	rjmp	TIM1_OVF	;Timer 1 overflow
;$007	
	rjmp	TIM0_OVF	;Timer 0 overflow
;$008	
	rjmp	SPI_Handler	;Serial transfer complete
;$009
	rjmp	UART_RXC	;Uart RX complete
;$00A
	rjmp	UART_DRE	;Uart data register empty
;$00B
	rjmp	UART_TXC	;Uart TX complete
;$00C	
	rjmp	ANA_COMP	;Analog comparator
;
;********
;
;All my ISRs use dedicated registers TTEMP and TTEMP2 to maximize speed. 
;You could push TEMP, and pop it back if you really need registers.
;
;Each int vector has, at minimum, code that turns off that int. 
;This is a "belts and suspenders" method that protects me from
;any ints I forgot to turn off, or accidentally enabled.
;
;If you decide to use one of the disabled ints, enable it in the
;code when appropriate, and replace these "turn-off" handlers
;with whatever it is you want done.
;
;Each ISR saves SREG to TTEMP2, which is dedicated to this purpose.
;I could have pushed and popped it, but push and pop take two cycles,
;and in and out only take one. If you get squeezed for registers, then
;you can always do the pushpop dance. It's an extra 500nS at 8MHz
;
;MAKE ABSOLUTELY SURE YOU DO NOT USE "TEMP" IN ANY ISR! (unless you push and pop it!)
;
;
;********
;
;Only used with the ISR debug statements. Set to any free pin
;
.equ	Pip_Port=PORTA	;
.equ	Pip_DDR=DDRA	;
.equ	Pip_Bit=1	;
;
;********
;
;Unused at this point. 
;
;
EXT_INT0:	
	in	TTEMP2,SREG	;Saves the status register
	
	;Since I use dedicated TTEMP registers for the interrupts, this is
	;all I need to do before acting on the int.  At the end of the isr,
	;SREG is restored, and we exit. 

	;The following code is a simple diagnostic, that can help you debug
	;interrupts. As shown, you would uncomment the debug code, and get 
	;a pulse on a pin during an interrupt. The width of the pulse shows 
	;how much time the ISR takes (plus 450nS for the debug instructions)
	;You can also put multiple SBI/CBI instructions at the beginning of
	;the ISRs, so that you can tell which is firing, if you have multiple
	;ISRs enabled to debug.

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;


	;Since this int isn't used in the app, I just leave it with the default
	;handler, which turns off this int. I should never have come here in the
	;first place, since this int is supposed to be off. This is VERY cheap
	;protection.  Some people code only the ISRs that they want. This can 
	;have two bad effects. One, if all you have in the table is a RETI, then
	;a wild int can eat up a ton of CPU time.
	;Two, if you don't have at least the RETI handlers for each int, then 
	;the wild int will cause the execution of the next int handler in the 
	;table, which could be a debug nightmare.

INT0_OFF:
	in	TTEMP,GIMSK	;If done, then turn off the int!
	andi	TTEMP,$BF	;
	out	GIMSK,TTEMP	;

INT0_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti
;
;********
;
;
EXT_INT1:
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

INT1_OFF:
	in	TTEMP,GIMSK	;Get the int enables
	andi	TTEMP,$7F	;Mask off INT1
	out	GIMSK,TTEMP	;Send it
	ldi	TTEMP,$80	;
	out	GIFR,TTEMP	;Manual clear
	
INT1_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;*******
;
;
TIM1_CAPT:			;
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

TIM1_OFF:
	in	TTEMP,TIMSK	;Shut off this int
	andi	TTEMP,$F7	;
	out	TIMSK,TTEMP	;

TIM1_EXIT:
	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti
;
;*********
;
TIM1_COMPA:			;
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

TC1A_OFF:
	in	TTEMP,TIMSK	;
	andi	TTEMP,$BF	;
	out	TIMSK,TTEMP	;

TC1A_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti			;
;	
;********
;
;
TIM1_COMPB:		
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

TC1B_OFF:
	in	TTEMP,TIMSK	;
	andi	TTEMP,$DF	;
	out	TIMSK,TTEMP	;

TC1B_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;*********
;
;Timer 1 gives us servo output timings from 1mS to 2mS, and allows
;the CPU to go do other things, rather than sitting there spinning 
;NOPS. It's a 16 bit counter, so I start it with a base value of servo_min,
;plus the servo width control byte (SERVO_X) We then have pulses that range
;from servo_min to servo_max with 8 bit resolution.
;
;Each servo out is activated sequentially in the main code. The activation
;code loads a value into Timer1, and sets the prescaler to /8, and sets the
;desired servo bit high. This int occurs when the servo's time expires, and
;we just shut off the output, and stop the timer. (Fast eh? :)
;
TIM1_OVF:
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;high


TIM1O_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2		;restore the status register
	reti				;
;
;*********
;
;	Main 1mS Opsys tick
;
; This seems like a huge routine, but actually, very little of it gets executed
; on any particular int. The main thing that we do, is decrement to zero any 
; timer bytes that aren't zero yet, and provide the state machine for the morse 
; code output that may have been started in MORSE.ASM
;
; There is also support for a timed wait between serial output chars, and 
; handlers for hardware or Xon/Xoff handshake.
;
TIM0_OVF:

	in	TTEMP3,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;Just high

	;**************
	;Reload timer 0 now to preserve accuracy
	;**************
	ldi	TTEMP,T0DIV	;Reload early, avoid the latency
	out	TCNT0,TTEMP	;


	;***********
	;EEPROM timer support
	;***********
T0_EEPROM:
	lds	TTEMP,EEPROM_Timer	;
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_EE_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	EEPROM_Timer,TTEMP ;and store
T0_EE_Done:	;Dummy label


	;***********
	;VFD Display scroll timer support
	;***********
T0_VFD	:
	lds	TTEMP,VFD_Timer	;
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_VFD_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	VFD_Timer,TTEMP	;and store
T0_VFD_Done:	;Dummy label


	;*******
	;Support for the timed watchdog handler
	;*******

T0_Dog:	lds	TTEMP,Dog_Food	;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_Dog_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Dog_Food,TTEMP	;and store
T0_Dog_Done:

	;*******

TIM0_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP3	;restore the status register
	reti			;Bye!
;
;*********
;
;SPI is unused at this point
;
SPI_Handler:			;
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

SPI_OFF:
	ldi	TTEMP,$3F	;Disable int, disable SPI
	out	SPCR,TTEMP	;

SPI_EXIT:
	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************
;
;Take a char and put it in the Serial input buffer. 
;Takes about 8uS due to handshaking logic
;
;
UART_RXC:			;
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;Two pings then high


	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	sreg,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Ready to transmit the next byte
;
;This takes about 10uS at 8 MHz, because of the handshake logic
;
UART_DRE:			;
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;Signal beginning of ISR



UART_DRE_EXIT:

	;Debug
	;cbi	Pip_Port,Pip_Bit;
	
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Uart TXC is unused at this point
;
UART_TXC:
	in	TTEMP2,SREG	;Saves the status register


	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;


UTXC_OFF:
	in	TTEMP,UCR	;
	andi	TTEMP,$BF	;Disable the TXC int.
	out	UCR,TTEMP	;

UTXC_EXIT:
	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Analog comparator is unused at this point.
;
ANA_COMP:
	in	TTEMP2,SREG	;Saves the status register

	;DEBUG
	;sbi	Pip_DDR,Pip_Bit	;Make it an output
	;sbi	Pip_Port,Pip_Bit;

ANAC_OFF:
	ldi	TTEMP,$83	;Shut off power to the comparator
	out	ACSR,TTEMP	;and the comparator ints

ANAC_EXIT:

	;DEBUG
	;cbi	Pip_Port,Pip_Bit;

	out	SREG,TTEMP2	;restore the status register
	reti
